
/************************************************************************/
/*                                                                      */
/*	Functions dealing with FATs - FATUTIL.C   July,  1987		*/
/*                                                                      */
/*	Version 1.0, July   1987					*/
/*	Version 1.2, August 1987					*/
/*                                                                      */
/*	Compiled under Lattice C version 3.20	(Large model -mls)	*/
/*                                                                      */
/*	Author: Lawrence E. Himes					*/
/*                                                                      */
/*	Rev 1.2 -  getmem() calls changed to malloc() for portability.	*/
/*                                                                      */
/************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "dglobal.h"

/****************************************************************/
/*                                                              */
/*	loadfat  () - Load up the all FATS, check equal, etc	*/
/*                                                              */
/****************************************************************/

loadfat ()
{
  int r, n, s, x;
  unsigned *fat [2];

	FAT = fat [1] = NULL;
	for (x = n = 0, s = resrvs; n < nfats ; n++, s+=Fatsecs  )  {
		if (n < 2)  {
			fat [n] = (unsigned *) (malloc (Fatsize));
			if (!(fat [n])) { r = Nomem; break; }
		}
		r = absdio (0, drive, fat [x], s, Fatsecs);
		if (r)	break;
		x = 1;
		if (n) {
			if (memcmp (fat [0], fat [1], Fatsize)) {
				r = Fatcomp; break;
			}
		}
	}
	if (fat [1]) free  ((char *)fat [1]);
	if (!r) { FAT = fat [0];
                if (!Algo) {
			Cfat = (char *)(FAT);
			Cutoff = 0x0FF8;
		 }
	}
	return r;
}

/****************************************************************/
/*                                                              */
/*	Getfate  () - Get FAT entry for specified cluster	*/
/*                                                              */
/****************************************************************/

unsigned getfate ( cluster)
  unsigned cluster;
{
  unsigned *temp;

	if (Algo) return (FAT [cluster]);	/* 16-bit FATs */
	cluster *= 3;				/* Get cluster offset */
	temp = (unsigned *)(Cfat + (cluster >> 1)); /* Load Word ptr */
	return (cluster & 1) ? ((*temp) >> 4) :
			       (*temp & 0xFFF);
}

/****************************************************************/
/*                                                              */
/*	Putfate  () - Put FAT entry for specified cluster	*/
/*                                                              */
/****************************************************************/

void putfate (fate, cluster)
  unsigned fate;
  unsigned cluster;
{
  unsigned  *fatp;

	if (Algo)				/* 16-bit FATs */
		FAT [cluster] = fate;
	else {
		cluster *= 3;			/* Get cluster offset */
		fatp = (unsigned *)(Cfat + (cluster >> 1));  /* Load Word ptr */
		if (cluster & 1)  {
			*fatp &= 0x000F;
			*fatp |= (fate << 4);
		} else {
			*fatp &= 0xF000;
			*fatp |= fate;
		}
	}
}

/****************************************************************/
/*                                                              */
/*	Cvtctos  () - Get base sector number given Cluster #	*/
/*                                                              */
/****************************************************************/

unsigned cvtctos (cluster)
  unsigned cluster;
{
	cluster -= 2;
	return (Ffilesec + (cluster * clsecs));
}

/****************************************************************/
/*                                                              */
/*	Cvtstoc  () - Get cluster number given sector # 	*/
/*                                                              */
/****************************************************************/

unsigned cvtstoc (sector )
  unsigned sector;
{
	sector -= Ffilesec;
	return (2 + (sector / clsecs));
}

/****************************************************************/
/*                                                              */
/*	dumpfat  () - Write out one or all FATS 		*/
/*                                                              */
/****************************************************************/

dumpfat (x)
  int x;
{
  int r, n, s;

	x = (x) ? nfats : 1;
	for (n = 0, s = resrvs; n < x ; n++, s+=Fatsecs  )  {
		r = absdio (1, drive, FAT, s, Fatsecs);
		if (r)	break;
	}
	return r;
}


/****************************************************************/
/*                                                              */
/*	rdcluster () - Read specified cluster to memory 	*/
/*                                                              */
/****************************************************************/

rdcluster (clu, buf)
  unsigned clu;
  char *buf;
{
  int  r;
  unsigned  s;

	s = cvtctos (clu);
	r = absdio (0, drive, buf, s, clsecs );
	return r;
}

/****************************************************************/
/*                                                              */
/*	wtcluster () - Write specified cluster from memory	*/
/*                                                              */
/****************************************************************/

wtcluster (clu, buf)
  unsigned clu;
  char *buf;
{
  int  r;
  unsigned  s;

	s = cvtctos (clu);
	r = absdio (1, drive, buf, s, clsecs );
	return r;
}


